<?php
/**
 * Joomla! 1.5 component Cu3er
 *
 * @version $Id: cu3erfile.class.php 2011-04-12 05:17:53 svn $
 * @author MADEBYPLAY
 * @package Joomla
 * @subpackage Cu3er
 * @license GNU/GPL
 *
 * Cu3er component for Joomla 1.5
 *
 * This component file was created using the Joomla Component Creator by Not Web Design
 * http://www.notwebdesign.com/joomla_component_creator/
 *
 */

// no direct access
defined('_JEXEC') or die('Restricted access');

/**
 * Cu3er Helper
 *
 * @package Joomla
 * @subpackage Cu3er
 * @since 1.5
 */
class Cu3erFile {

	/**
	 * Checks if the file can be uploaded
	 *
	 * @param array $file File information
	 * @param string $err An error message to be returned
	 * @param string $allowed allowed (comma separated) file extensions
	 * @return boolean
	 */
	function canUpload( $file, &$err, $allowed)
	{
		// if there is no filename
		if(empty($file['name'])) {
			$err = JText::_('Please input a file for upload');
			return false;
		}

		jimport('joomla.filesystem.file');
		// warning of a not safe file naming
		if ($file['name'] !== JFile::makesafe($file['name'])) {
			$err = 'WARNFILENAME';
			return false;
		}

		// getting file extension
		$ext = strtolower(JFile::getExt($file['name']));

		// generating array of allowed file extensions
		$allowable = explode(",", $allowed);

		// handling not allowed file extension error
		if (!in_array($ext, $allowable))
		{
			$err = JText::_('File type is not allowed') . ' (' . $allowed . ')';
			return false;
		}

		return true;
	}

	/**
	 * File upload handler method
	 *
	 * @access	public
	 * @param 	mixed $return		Redirect query string
	 * @param 	string $ext		Allowed (comma separated) file extensions
	 * @param 	boolean $overwrite	true if overwrite is enabled
	 * @param 	mixed $file		file data array
	 * @param 	mixed $filepath		destination path of file
	 * @param 	boolean $redirect	redirect on success
	 */
	function uploadFile($return, $ext = 'zip,xml', $overwrite = false, $file = false, $filepath = false, $redirect = true) {

		$mainframe = &JFactory::getApplication();

		// setting file vars
		$file 		= ($file)?($file):JRequest::getVar('cu3er_file', '', 'files', 'array');
		$filepath 	= ($filepath)?($filepath.DS.$file['name']):(JPath::clean(JPATH_COMPONENT.DS.'upload'.DS.$file['name']));
		$err		= null;

		// setting FTP credentials, if given (write permission purposes)
		jimport('joomla.client.helper');
		JClientHelper::setCredentialsFromRequest('ftp');

		$file['name']	= JFile::makeSafe($file['name']);

		if (isset($file['name'])) {

			// handling upload errors
			if (!Cu3erFile::canUpload($file, $err, $ext)) {
				JError::raiseNotice(100, JText::_($err));
				$mainframe->redirect($return);
			}

			// if uploaded package file exists
			if (JFile::exists($filepath) && !$overwrite) {
				JError::raiseNotice(100, JText::_('Error. File already exists'));
				$mainframe->redirect($return);
			}

			// handling upload errors
			if (!JFile::upload($file['tmp_name'], $filepath)) {
				JError::raiseWarning(100, JText::_('Error. Unable to upload file'));
				$mainframe->redirect($return);
			} else {				
				if ($redirect) {			
					$mainframe->enqueueMessage(JText::_('Upload complete'));
					$mainframe->redirect($return.'&state=1&filepath='.$filepath);
				} else {
					return true;				
				}
			}
		} else {
			$mainframe->redirect(base64_decode($return), 'Invalid Request', 'error');
		}	
	}

	/**
	 * URI checker method
	 *
	 * @param string $uri URI string
	 * @return mixed boolean, or array containing socket_read outpout and status message
	 */
	function lookup($uri) {
		
		// setting up http status codes	
		$status = array ( 
			100 => 'Continue', 101 => 'Switching Protocols', 200 => 'OK', 201 => 'Created', 202 => 'Accepted', 
			203 => 'Non-Authoritative Information', 204 => 'No Content', 205 => 'Reset Content', 206 => 'Partial Content', 
			300 => 'Multiple Choices', 301 => 'Moved Permanently', 302 => 'Found', 303 => 'Other', 304 => 'Not Modified', 
			305 => 'Use Proxy', 307 => 'Temporary Redirect', 400 => 'Bad Request', 401 => 'Unauthorized', 402 => 'Payment Required',
			403 => 'Forbidden', 404 => 'Not Found', 405 => 'Method Not Allowed', 406 => 'Not Acceptable', 407 => 'Proxy Authentication Required', 
			408 => 'Request Timeout', 409 => 'Conflict', 410 => 'Gone', 411 => 'Length Required', 412 => 'Precondition Failed', 
			413 => 'Request Entity Too Large', 414 => 'Request-URI Too Long', 415 => 'Unsupported Media Type', 416 => 'Requested Range Not Satisfiable', 
			417 => 'Expectation Failed', 500 => 'Internal Server Error', 501 => 'Not Implemented', 502 => 'Bad Gateway', 
			503 => 'Service Unavailable', 504 => 'Gateway Timeout', 505 => 'HTTP Version Not Supported' 
    		); 
   		
		// parsing given URL
		$u = parse_url($uri); 

		// setting root path    		
		if(!isset($u['path'])) { 
        		$u['path'] = '/'; 
    		}

		// setting default port
    		if(!isset($u['port'])) { 
        		$u['port'] = getservbyname('www', 'tcp'); 
    		} 
    
		// setting host
		$addr = ($u['host'] == 'localhost')?'127.0.0.1':gethostbyname($u['domain']); 

		// creating socket
		$sp = @socket_create(AF_INET, SOCK_STREAM, SOL_TCP); 

		if(!$sp) { 
        		return false; 
    		} 

		socket_set_timeout($sp, 0, 0); 
   		$result = @socket_connect($sp, $addr, $u['port']); 

    		if(!$result) { 
			return false; 
    		}
 
		// http header string		
		$in = "HEAD {$u['path']} HTTP/1.1\r\nHost: {$u['domain']}\r\nConnection: Close\r\n\r\n"; 

		// socket writing		
		socket_write($sp, $in, strlen($in)); 

		// socket reading
		$out = socket_read($sp, 1024); 
		socket_close($sp); 
		
		// setting response array
		$out = explode("\r\n", $out, 2); 
		unset($out[1]); 
		$out = explode(" ", $out[0]); 
    		return array($out[1], $status[$out[1]]);
	}

	/**
	 * Recursive directory and file removal method
	 *
	 * @param string $directory directory name
	 * @param boolean $empty if true the directory given in the $directory variable will not be deleted
	 * @return boolean true if success
	 */
	function deleteAll($directory, $empty = false) { 
		if(substr($directory,-1) == DS) { 
			$directory = substr($directory,0,-1); 
		} 

		if(!file_exists($directory) || !is_dir($directory)) { 
			return false; 
		} elseif(!is_readable($directory)) { 
			return false; 
		} else { 
			$directoryHandle = opendir($directory); 
        
			while ($contents = readdir($directoryHandle)) { 

				if($contents != '.' && $contents != '..') { 
					$path = $directory . DS . $contents; 
                
					if(is_dir($path)) { 
						Cu3erFile::deleteAll($path); 
					} else { 
						unlink($path); 
					} 
				}
			} 	
		} 
        
		closedir($directoryHandle); 

		if($empty == false) { 
			if(!rmdir($directory)) { 
				return false; 
			} 
		} 
        
		return true; 
	}

	/**
	 * Remote file size getter method
	 *
	 * @param string	$url URL of the remote file
	 * @return int		size of remote file in bytes
	 */	
	function getRemoteFileSize($url) {
		$parsed = parse_url($url); 
		$host = $parsed["host"]; 
		$fp = @fsockopen($host, 80, $errno, $errstr, 20); 
			
		if(!$fp) {
			return false; 
		} else { 
			@fputs($fp, "HEAD $url HTTP/1.1\r\n"); 
			@fputs($fp, "HOST: $host\r\n"); 
			@fputs($fp, "Connection: close\r\n\r\n"); 
			$headers = ""; 

			while(!@feof($fp)) {
				$headers .= @fgets($fp, 128);
			} 
   		}

		@fclose ($fp); 
		$return = false;

		$arr_headers = explode("\n", $headers); 

		foreach($arr_headers as $header) { 
			$s = "Content-Length: "; 
			if(substr(strtolower ($header), 0, strlen($s)) == strtolower($s)) { 
				$return = trim(substr($header, strlen($s))); 
				break; 
			} 
		} 
	
   		return (int)$return;
	}

	/**
	 * Remote file saver method
	 *
	 * @param string $url URL of the file
	 * @param string $dest_path destination path of the file
	 * @return boolean true if success
	 */	
	function saveRemoteFile($url, $dest_path) {
		jimport('joomla.filesystem.file');
		// reading URL into string		
		$file_content = JFile::read($url);
		// writing string into file
		return JFile::write($dest_path, $file_content);
	}

	/**
	 * Custom XML file generator method
	 *
	 * @param string  $slideshow_name name of the slideshow
	 * @param int 	  $slideshow_id ID of the slideshow
	 * @param boolean $published true if slideshow is published
	 * @param array   $prev_array array of modified parameters for preview
	 * @return boolean true if success
	 */	
	function generateXMLFile($slideshow_id, $published = true, $prev_array = null) {
	
		jimport('joomla.filesystem.file');

		require_once(JPATH_COMPONENT_ADMINISTRATOR.DS.'lib'.DS.'cu3erxml.class.php');
		// instantiating Cu3erXML class
		$cxml = new Cu3erXML($slideshow_id);

		// generating XML string
		$XML_encoded = $cxml->generateXML($slideshow_id, $prev_array);

		// writing XML string into CU3ER-generated.xml if task is not preview
		if ($prev_array === NULL) {
			JFile::write(JPATH_COMPONENT_SITE.DS.'slideshows'.DS.$slideshow_id.DS.'CU3ER-generated.xml', $XML_encoded);
		}

		// writing XML string into CU3ER-preview.xml 
		return (JFile::write(JPATH_COMPONENT_SITE.DS.'slideshows'.DS.$slideshow_id.DS.'CU3ER-preview.xml', $XML_encoded));
	}

	/**
	 * CU3ER javascript file includer method
	 *
	 * @param string  $filename name of the javascript file
	 * @return boolean true if success
	 */	
	function addJsFile($filename) {
		$doc = & JFactory::getDocument();
		// getting scripts from document's head		
		$header = $doc->getHeadData();

		// if $filename not exists in the head (adding only once)
		if (preg_match('/$filename/', implode("#", array_keys($header['scripts']))) == 0) {
			$doc->addScript(JURI::base().'components/com_cu3er/js/'.$filename);
		}
		return true;
	}
	
	function writeSession($name, $value) {
		$app = & JFactory::getApplication();
		$app->set($name, $value);
		return true;
	}

	/**
	 * CU3ER CSS file includer method
	 *
	 * @param string  $filename name of the CSS file
	 * @param mixed	  $condition conditional comment string or false by default
	 * @return boolean true if success
	 */	
	function addCSSFile($filename, $condition = false) {
		$doc = & JFactory::getDocument();			
		// getting scripts from document's head		
		$header = $doc->getHeadData();
		// if $filename not exists in the head (adding only once)
		if (preg_match('/$filename/', implode("#", array_keys($header['scripts']))) == 0) {
			// generating conditional commented HTML (eg ie compatibility)			
			if ($condition !== false) {			
				$stylelink = '<!--[if '.$condition.']>' ."\n";
				$stylelink .= '<link rel="stylesheet" href="'.JURI::base().'components/com_cu3er/js/'.$filename.'" />' ."\n";
				$stylelink .= '<![endif]-->' ."\n";
				$doc->addCustomTag($stylelink);
			} else {
				$doc->addStyleSheet(JURI::base().'components/com_cu3er/css/'.$filename);
			}
		}
		return true;
	}

}
?>