<?php
/**
* @version $Id: cu3er.php v 1.1 2010/04/22 $
* @package Joomla! 1.5
* @copyright (c) 2011 MADEBYPLAY
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL
*
* Usage
* {CU3ER slideshowID}
*
*/

defined('_JEXEC') or die( "Direct Access Is Not Allowed" );

// importing library dependencies
jimport('joomla.event.plugin');

class plgContentCu3er extends JPlugin
{
	var $document;
	var $plugin_params;
	var $alt_content;
	var $js_str;
   
	/**
	 * Constructor
	 * 
	 * @access	public
	 */
	function plgContentCu3er(&$subject) {
		parent::__construct( $subject );
		// load plugin parameters
		$this->_plugin = JPluginHelper::getPlugin( 'content', 'cu3er' );
		$this->_params = new JParameter( $this->_plugin->params );
		$this->document = &JFactory::getDocument();		
    	}
	/**
	 * Plugin method with the same name as the event will be called automatically
	 * 
	 * @access	public
	 * @param	object	 $article	a reference to the article that is being rendered by the view
	 * @param	array	 $params	a reference to an associative array of relevant parameters
	 * @param	int	 $limitstart	determines the "page" of the content that is to be generated
	 */
	function onPrepareContent(&$article, &$params, $limitstart) {
		// just startup

		// including CU3ER file handler class		
		require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_cu3er'.DS.'lib'.DS.'cu3erfile.class.php');

		$plugin =& JPluginHelper::getPlugin('content', 'cu3er');
		$this->plugin_params = new JParameter( $plugin->params );

		// expression to search for
		$regex = '/{(CU3ER)\s*(.*?)}/i';
		
		// finding all instances of $regex (i.e. {CU3ER .. }) in an article and put them in $matches
		$matches = array();
		preg_match_all( $regex, $article->text, $matches, PREG_SET_ORDER );      
		
		// including javascript files into the HTML head if not exist (just once)
		Cu3erFile::addJsFile('jquery.js');
		Cu3erFile::addJsFile('swfobject.js');
		Cu3erFile::addJsFile('jquery.cu3er.js');
		
		// counter for CU3ER instances
		$num = 1;

		// cycling through all instances
		foreach ($matches as $match) {
			
			$id = $match[2];

			// if slideshow is published
			if ($this->generateJS($id, $num)) {
				// adding javascript embedder code to HTML head				
				$this->document->addScriptDeclaration($this->js_str);
			}

			// setting alternative content
			$alt_content = $this->alt_content;

			// including CU3ER container template
			ob_start();
			include('cu3er/tmpl/default.php');
			
			// fixing Joomla bug 
			$output_container = str_replace( '$' , '\$' , ob_get_contents());
			ob_end_clean();

			// final replacing of $regex (i.e. {CU3ER .. }) in $article->text by $output
			$article->text = preg_replace($regex, $output_container, $article->text, 1);
			
			// stepping the CU3ER instance counter
			$num++;
		}
		 
		return true;   
	}
	
	/**
	 * Plugin method with the same name as the event will be called automatically
	 * 
	 * @access	public
	 * @param	object	 $article	a reference to the article that is being rendered by the view
	 * @param	array	 $params	a reference to an associative array of relevant parameters
	 * @param	int	 $limitstart	determines the "page" of the content that is to be generated
	 */
	function generateJS($id, $num) {
	
		jimport('joomla.filesystem.file');
		jimport('joomla.utilities.simplexml');

		// reading XML encoded string
		$XML_encoded = JFile::read(JPATH_SITE.DS.'components'.DS.'com_cu3er'.DS.'slideshows'.DS.$id.DS.'CU3ER-generated.xml');

		// parsing XML string
		$xml = JFactory::getXMLParser('Simple');
		$xml->loadString($XML_encoded);

		// referencing XML document
		$xml_doc = &$xml->document;

		// if slideshow is published		
		if (	$xml_doc->getElementByPath('settings/published') && 
			$xml_doc->getElementByPath('settings/published')->data() == 1) {

			// setting alternative content for embedding
			$this->alt_content = $xml_doc->getElementByPath('settings/content')->data();

			// setting slideshow parameters from parsed XML nodes
			$backgroundType = ($xml_doc->getElementByPath('settings/background/color')->attributes('transparent'))?'transparent':false;		
			$background = $xml_doc->getElementByPath('settings/background/color')->data();
			$folder_images = $xml_doc->getElementByPath('settings/folder_images')->data();
			$folder_fonts = $xml_doc->getElementByPath('settings/folder_fonts')->data();
			$width = $xml_doc->getElementByPath('project_settings/width')->data();
			$height = $xml_doc->getElementByPath('project_settings/height')->data();
			$force_js = ($xml_doc->getElementByPath('settings/force_js'))?$xml_doc->getElementByPath('settings/force_js')->data():'false';

			// adding neccessary js and css files to HTML head for displaying javascript version (only once)
			if ($force_js != 'false') {
				Cu3erFile::addJsFile('jquery.cu3er.player.js');
				Cu3erFile::addCSSFile('CU3ER.css');
				Cu3erFile::addCSSFile('CU3ERieFix.css', 'IE');
			}

			$CU3ER_root = JURI::root().'components'.DS.'com_cu3er';
			// stripping http://
			preg_replace('/http\:\/\/(.*?)\//', '/', $CU3ER_root);		

			// setting wmode parameter
			$wmode = ($backgroundType == 'transparent') ? "transparent" : "color: ".str_replace(array('0x', '0X'), array('#', '#'), $background);

			// URL encoding XML string
			$uenc_XML = urlencode($XML_encoded);

			// setting CU3ER embedder javascript code
			$js_str = <<<EOD
				
				jQuery.noConflict();
				jQuery(document).ready(function($) {					
					$('#CU3ERplg{$num}').cu3er({
						vars: {
							xml_location: '{$CU3ER_root}/slideshows/{$id}/CU3ER-config.xml',
							xml_encoded: '{$uenc_XML}',
							swf_location: '{$CU3ER_root}/flash/CU3ER.swf',
							width: '{$width}',
							height: '{$height}',
							force_javascript: {$force_js},
							folder_images: '{$folder_images}',
							folder_fonts: '{$folder_fonts}',
						},
						params: {
							wmode: '{$wmode}',
							allowScriptAccess: 'always',
						},
						attributes: {
							name:'CU3ERplg{$num}',
							id:'CU3ERplg{$num}',
						}
					});
				});
EOD;

			// setting js_str property (javascript embedder code string)
			return ($this->js_str = $js_str);
		} else {
			return false;
		}
	}

}
?>